"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.visitCallingArguments = void 0;
const typescript_1 = __importDefault(require("typescript"));
const utils_1 = require("./utils");
const visit_fn_res_1 = require("./visit-fn-res");
/**
 * Visit the arguments of a function call. If the argument is a FnResource, it will be extracted as
 * a closure.
 * @param signature The signature of the function.
 * @param args The arguments of the function call.
 * @param checker The type checker.
 * @returns
 */
function visitCallingArguments(signature, args, checker) {
    const closureInfos = []; // Record the closure list for this call.
    const closureDependencies = []; // Record the access relationship information that is located in the body of FnResource.
    const parameters = []; // Record the argument information for this call.
    // If the arguments is undefined, it means that this function call does not have any arguments.
    if (!args) {
        return {
            closureInfos: closureInfos,
            closureDependencies: closureDependencies,
            parameters: parameters,
        };
    }
    signature.parameters.forEach((paramSig, idx) => {
        const paramName = paramSig.name;
        const arg = args[idx];
        const param = { name: paramName, order: idx, expression: arg };
        const paramType = checker.getTypeOfSymbol(paramSig);
        const decls = paramType.symbol?.declarations;
        if (decls != undefined &&
            decls.length >= 1 &&
            (typescript_1.default.isInterfaceDeclaration(decls[0]) || typescript_1.default.isClassDeclaration(decls[0])) &&
            (0, utils_1.isResourceType)(decls[0], checker, true)) {
            // This parameter type is FnResource. Construct a resource.
            if (decls.length != 1) {
                console.warn("Found a parameter with more than one declarations: " + decls[0].getText());
            }
            // Generate the fn resource name
            let closureName = "";
            if (typescript_1.default.isFunctionExpression(arg)) {
                closureName = arg.name?.getText() ?? "";
            }
            if (closureName == "") {
                const { line, character } = typescript_1.default.getLineAndCharacterOfPosition(arg.getSourceFile(), arg.getStart());
                closureName = `fn_${line + 1}_${character + 1}`;
            }
            const result = (0, visit_fn_res_1.visitFnResourceBody)(arg, checker, closureName);
            if (result.resourceVarInfos) {
                closureInfos.push(...result.resourceVarInfos);
            }
            if (result.resourceRelatInfos) {
                closureDependencies.push(...result.resourceRelatInfos);
            }
            param.resourceName = closureName;
            parameters.push(param);
            return;
        }
        parameters.push(param);
        return;
    });
    return {
        closureInfos: closureInfos,
        closureDependencies: closureDependencies,
        parameters: parameters,
    };
}
exports.visitCallingArguments = visitCallingArguments;
//# sourceMappingURL=visit-calling-arguments.js.map